@@ -1,8 +1,14 @@
-use voltdb_client_rust::node::*;
-use voltdb_client_rust::encode::VoltError;
+use std::fs;
+use std::time::SystemTime;
+
 use bigdecimal::BigDecimal;
+use bigdecimal::num_bigint::BigInt;
 use chrono::{DateTime, Utc};
+
+use voltdb_client_rust::encode::*;
+use voltdb_client_rust::node::*;
 use voltdb_client_rust::table::VoltTable;
+use voltdb_client_rust::volt_param;
 
 fn main() -> Result<(), VoltError> {
     #[derive(Debug)]
@@ -73,12 +79,32 @@ fn main() -> Result<(), VoltError> {
         let test: Test = table.map_row();
         println!("{:?}", test);
     }
-    let insert_value = "insert into test_types (T1,T2,T3,T4,T5,T6,T7,T8,T9) values (1,2,3,4,5,6,'7','089CD7B35220FFB686012A0B08B49ECD8C06109893971F422F4D4F4E49544F52494E475F33393766643034662D656161642D346230372D613638302D62663562633736666132363148D535A8019CD7B352B001DDEE8501B801BAEE8501C001AAE98601CA01054341534831D0010AE00102E80102F20103555344FA010A0A0355534410809BEE028202050A035553448A020B08B49ECD8C06109893971F9202046E756C6CA2020A0A0355534410C0BD9A2FBA0219312C323139313936382C323139333231302C32313933323435C802C91E8A0400920400D80401880505B20500',NOW());";
-    block_for_result(&node.query(insert_value)?)?;
-    let mut table = block_for_result(&node.query("select * from test_types")?)?;
+    let bs = vec![1 as u8, 2, 3, 4];
+    let time = DateTime::from(SystemTime::now());
+    // call proc with parameters
+    let mut table = block_for_result(&node.call_sp("test_types.insert", volt_param![1,2,3,4,5,6,"7",bs,time])?)?;
     while table.advance_row() {
-        let test: Test = table.map_row();
-        println!("{:?}", test);
+        println!("{}", table.debug_row());
+    }
+    // upload proc into server
+    let jars = fs::read("tests/procedures.jar").unwrap();
+    let x = node.upload_jar(jars).unwrap();
+    let mut table = x.recv().unwrap();
+    assert!(table.has_error().is_none());
+    // create sp
+    let script = "CREATE PROCEDURE  FROM CLASS com.johnny.ApplicationCreate;";
+    block_for_result(&node.query(script)?);
+    let header = vec!["T1", "T2", "T3", "T4", "T5", "T6", "T7", "T8", "T9"];
+    let tp = vec![TINYINT_COLUMN, SHORT_COLUMN, INT_COLUMN, LONG_COLUMN, FLOAT_COLUMN, DECIMAL_COLUMN, STRING_COLUMN, VAR_BIN_COLUMN, TIMESTAMP_COLUMN];
+    let header: Vec<String> = header.iter().map(|f| f.to_string()).collect::<Vec<String>>();
+    let mut table = VoltTable::new_table(tp, header);
+    let decimal = BigDecimal::from(16);
+    let data = volt_param! {true, 12 as i8 , 13 as i16,  14 as i32,  15.0, decimal  , "17",bs, time   };
+    table.add_row(data);
+    // call proc with volt table
+    let mut res = block_for_result(&node.call_sp("ApplicationCreate", volt_param![table])?)?;
+    while res.advance_row() {
+        println!("{:?}", res.debug_row());
     }
     Ok({})
 }
